<?php

/**
 * Class POP3
 *
 * Receive email from POP3 or IMAP server using the PHP IMAP extension
 * For simplicity, all returned strings are encoded as UTF-8
 *
 * Last changes:
 * 2010-06-27 Created
 *
 * @author Valentin Schmidt
 * @package Email
 * @version 0.1.1
 */
class POP3{

var $host;
var $port;
var $protocol;
var $sslFlag;
var $username;
var $password;

/**
 * Sets POP3/IMAP configuration
 *
 * @access public
 * @param string $host
 * @param int $port 'pop3' or 'imap'
 * @sslFlag bool default=false
 * @param string $username optional
 * @param string $password optional
 * @return void
 */
#public 
function setConfig($host='localhost', $port=110, $protocol='pop3', $sslFlag=false, $username=null, $password=null){
	$this->host     = $host;
	$this->port     = $port;
	$this->protocol = $protocol;
	$this->sslFlag  = $sslFlag;
	$this->username = $username;
	$this->password = $password;
}

/**
 * Return number of (new) messages on POP3/IMAP server
 *
 * @access public
 * @return false or int number of messages or false
 * (@throws Could not connect ... 1)
 */
function getStatus(){
	$mbox = imap_open('{'.$this->host.':'.$this->port.'/'.$this->protocol.($this->sslFlag?'/ssl':'').'/novalidate-cert}INBOX', $this->username, $this->password);
	if ($mbox===false){
		#throw new Exception('Could not connect to host: '.imap_last_error(), 1); # PHP5
		$err = imap_errors(); # clear error buffer
		return false; # PHP4
	}
	$info = imap_check($mbox);
	$err = imap_errors(); # clear error buffer
	imap_close($mbox);
	return $info->Recent; # Nmsgs
}

/**
 * Return array of emails from POP3/IMAP server
 *
 * @access public
 * @param string $to
 * @param string $subject
 * @param string $message
 * @param array $headers optional
 * @return false or array emails as array of $message_number=>$email
 * where each $email is an array with the following keys: 
 * - headers object raw list of headers as returned by imap_headerinfo()
 * - date int timestamp
 * - subject string (utf-8)
 * - from_name string (utf-8)
 * - from_email string 
 * (@throws Could not connect ... 1)
 */
#public 
function getList(){
	$emails = array();
	$mbox = imap_open('{'.$this->host.':'.$this->port.'/'.$this->protocol.($this->sslFlag?'/ssl':'').'/novalidate-cert}INBOX', $this->username, $this->password);
	if ($mbox===false){
		#throw new Exception('Could not connect to host: '.imap_last_error(), 1); # PHP5
		$err = imap_errors(); # clear error buffer
		return false; # PHP4
	}
	$cnt = imap_num_msg($mbox);
	for ($n=1;$n<=$cnt;$n++){
		
		$mail = array();
		$header = imap_headerinfo($mbox, $n);
		$mail['headers'] = $header;
		
	  $structure = imap_fetchstructure($mbox, $n);
		$params = $this->getParams($structure->type==1 ? $structure->parts[0] : $structure);
		$is_utf8 = (@$params['CHARSET']=='UTF-8');
	
		$date = @$header->date;
		$ts = strtotime($date);
		$mail['date'] = $date;
		
		$subject = @$header->subject;
		$subject = imap_utf8($subject);
		if (!$is_utf8) $subject = utf8_encode($subject);
		$subject = htmlspecialchars($subject);
		$mail['subject'] = $subject;
		
		$fromaddress = @$header->fromaddress;
		$from_name = @$header->from[0]->personal;
		$from_name = imap_utf8($from_name);
		if (!$is_utf8) $from_name = utf8_encode($from_name);
		$from_email = @$header->from[0]->mailbox.'@'.$header->from[0]->host;
		$mail['from_name'] = $from_name;
		$mail['from_email'] = $from_email;
				
		$emails[$n] = $mail;
	}
	$err = imap_errors(); # clear error buffer
	imap_close($mbox);
	return $emails;
}

/**
 * Return email message with message_number $n
 *
 * @access public
 * @param int $to
 * @param bool $getBody default=true
 * @param bool $getAttachmentData default=false (if false, only filenames and sizes are returned)
 * @param bool $deleteFlag optional default=false
 * @return false or email as array with the following keys:
 * - headers object raw list of headers as returned by imap_headerinfo()
 * - body string (utf-8)
 * - attachments array
 * - is_html bool
 * - date int timestamp
 * - subject string (utf-8)
 * - from_name string (utf-8)
 * - from_email string
 * - reply_to_name string (utf-8)
 * - reply_to_email string
 * (@throws Could not connect ... 1)
 */
#public 
function getEmail($n, $getBody=true, $getAttachmentData=false, $deleteFlag=false){
	
	$mbox = imap_open('{'.$this->host.':'.$this->port.'/'.$this->protocol.($this->sslFlag?'/ssl':'').'/novalidate-cert}INBOX', $this->username, $this->password);
	if ($mbox===false){
		#throw new Exception('Could not connect to host: '.imap_last_error(), 1); # PHP5
		$err = imap_errors(); # clear error buffer
		return false; # PHP4
	}
	$mail = array();
	$attachments = array();
	
	$is_html = false;
	
	$header = imap_headerinfo($mbox, $n);
	$mail['headers'] = $header;

	$structure = imap_fetchstructure($mbox, $n);
	if($structure->type == 1){ # MULTI-PART MESSAGE
		
		if ($getBody){
			$body = imap_fetchbody($mbox, $n, '1');
			if ($structure->parts[0]->encoding==3) $body = base64_decode($body);
			elseif ($structure->parts[0]->encoding==4) $body = quoted_printable_decode($body);
		}
		if (strtoupper($structure->parts[0]->subtype)=='HTML') $is_html=true;

		$params = $this->getParams($structure->parts[0]);
		$is_utf8 = (@$params['CHARSET']=='UTF-8');
		
		$cnt = count($structure->parts);
		for ($i=1;$i<$cnt;$i++){
			$part = $structure->parts[$i];
			foreach ($part->parameters as $param){
				if ($param->attribute=='NAME'){					
					$attachments[$i] = array(
						'part' => $i,
						'filename' => $param->value,
						'size' => $part->bytes
					);
					if ($getAttachmentData){
						$data = imap_fetchbody($mbox, $n, (string)($i+1)); # +1 ?
						if ($part->encoding==3) $data = base64_decode($data);
						$attachments[$i]['data'] = $data;
					}
					break;
				}
			}
		}
		
	}else{

		$params = $this->getParams($structure);
		$is_utf8 = (@$params['CHARSET']=='UTF-8');
		
		if ($getBody){
			$body = imap_body($mbox, $n);
			if ($structure->encoding==3) $body = base64_decode($body);
			elseif ($structure->encoding==4) $body = quoted_printable_decode($body);
		}
		if ($structure->subtype=='HTML') $is_html=true;
	}
	
	$mail['attachments'] = $attachments;
	$mail['is_html'] = $is_html;
		
	$date = @$header->date;
	$ts = strtotime($date);
	$mail['date'] = $date;
	
	$subject = @$header->subject;
	$subject = imap_utf8($subject);
	if (!$is_utf8) $subject = utf8_encode($subject);
	$subject = htmlspecialchars($subject);
	$mail['subject'] = $subject;
	
	$fromaddress = @$header->fromaddress;
	$from_name = @$header->from[0]->personal;
	$from_name = imap_utf8($from_name);
	if (!$is_utf8) $from_name = utf8_encode($from_name);
	$from_email = @$header->from[0]->mailbox.'@'.$header->from[0]->host;
	$mail['from_name'] = $from_name;
	$mail['from_email'] = $from_email;

	$reply_to_address = @$header->reply_toaddress;
	$reply_to_name = @$header->reply_to[0]->personal;
	$reply_to_name = imap_utf8($reply_to_name);
	if (!$is_utf8) $reply_to_name = utf8_encode($reply_to_name);
	$reply_to_email = @$header->reply_to[0]->mailbox.'@'.$header->reply_to[0]->host;
	$mail['reply_to_name'] = $reply_to_name;
	$mail['reply_to_email'] = $reply_to_email;
	
	if ($getBody){
		$body = trim($body);
		if (!$is_utf8) $body = utf8_encode($body);
		$mail['body'] = $body;
	}
	if ($deleteFlag){
		imap_delete($mbox, $n);
		imap_expunge($mbox);
	}
	$err = imap_errors(); # clear error buffer
	imap_close($mbox);
	return $mail;
}

/**
 * Delete specified (or all) messages from server
 *
 * @access public
 * @param int|array|void $msg_nums
 * @return bool success
 * (@throws Could not connect ... 1)
 */
 #public
function deleteEmails($msg_nums=null){
	$mbox = imap_open('{'.$this->host.':'.$this->port.'/'.$this->protocol.($this->sslFlag?'/ssl':'').'/novalidate-cert}INBOX', $this->username, $this->password);
	if ($mbox===false){
		#throw new Exception('Could not connect to host: '.imap_last_error(), 1); # PHP5
		$err = imap_errors(); # clear error buffer
		return false; # PHP4
	}
	if (is_null($msg_nums)){
		@imap_delete($mbox,'1:*'); # to clear out an entire mailbox.
	}else{
		if (!is_array($msg_nums)) $msg_nums=array($msg_nums);
		rsort($msg_nums, SORT_NUMERIC);
		foreach ($msg_nums as $n){
			@imap_delete($mbox, $n);
		}
	}
	$err = imap_errors(); # clear error buffer
	imap_close($mbox, CL_EXPUNGE);
	return true;
}

/**
 * Converts parameters of IMAP structure to array with uppercase keys and values
 *
 * @access private
 * @param object $obj
 * @return array
 */
#private 
function getParams($obj){
	$params = array();
	$cnt = @count($obj->parameters);
	for ($i=0;$i<$cnt;$i++){
		$params[strtoupper($obj->parameters[$i]->attribute)] = strtoupper($obj->parameters[$i]->value);
	}
	return $params;
}

} // END CLASS

?>